// Project : K-means
// Author : Dario GENGA

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "kmeans.h"

#define REQUIRED_FULL_ARGS 5
#define ONLY_SOURCE_PATH_ARG 2
#define ONLY_SOURCE_AND_OUTPUT_PATH_ARGS 3
#define DEFAULT_FILE_OUTPUT "./output.txt"

kmeans *init_frm_cmd_arguments(int argc, char *argv[], char *output_file) {
    kmeans *universe;
    int k;
    int dimensions;
    char data_path[256];
    char output[256];
    int *data_length = malloc(sizeof(int));
    point** data;

    // No arguments, ask the user the settings and data
    if (argc < 2) {
        printf("Number of clusters (k) : ");
        scanf("%d", &k);
        printf("Number of dimensions for the data : ");
        scanf("%d", &dimensions);
        printf("Path to data (comma separator) : ");
        scanf("%s", data_path);
        printf("Output : ");
        scanf("%s", output);
    }
    // Create the universe with only our file
    else if (argc == ONLY_SOURCE_PATH_ARG || argc == ONLY_SOURCE_AND_OUTPUT_PATH_ARGS) {
        strcpy(data_path, argv[1]);
        universe = kmeans_create_empty();
        read_custom_data_source(universe, data_path);
        if (argc == ONLY_SOURCE_PATH_ARG)
            strcpy(output, DEFAULT_FILE_OUTPUT);
        else
            strcpy(output, argv[2]);
    }
    // Set the number of dimensions, clusters and path to data
    else if (argc == REQUIRED_FULL_ARGS) {
        dimensions = atoi(argv[1]);
        k = atoi(argv[2]);
        strcpy(data_path, argv[3]);
        strcpy(output, argv[4]);
    }
    // Bad number of arguments
    else {
        if (argc < REQUIRED_FULL_ARGS)
            printf("Missing arguments, ");
        if (argc > REQUIRED_FULL_ARGS)
            printf("Too much arguments, ");

        // Display instructions
        printf("you must start the program without arguments or with one of the following format :\n");
        printf("main <number of dimensions> <number of cluster> <path_to_data_source> <output_path>\n");
        printf("main <path_to_custom_data_source> <output_path>\n");
        printf("main <path_to_custom_data_source>\n");
        exit(EXIT_SUCCESS);
    }

    // Save the output file path
    strcpy(output_file, output);

    // Create the universe
    if (argc != ONLY_SOURCE_PATH_ARG && argc != ONLY_SOURCE_AND_OUTPUT_PATH_ARGS) {
        data = read_data(data_path, dimensions, data_length);
        universe = kmeans_create(k, dimensions, data, *data_length);
    }

    free(data_length);
    return universe;
}

int main(int argc, char *argv[]) {
    srand(time(NULL));
    char output_file[256];

    // Initialize the universe
    kmeans *universe = init_frm_cmd_arguments(argc, argv, output_file);
    printf("%s", output_file);

    // Start the clustering
    start_clustering(universe);

    // Output the result to a file
    write_data_output(universe, output_file);

    // Free the universe and exit the program
    destroy_universe(universe);
    return EXIT_SUCCESS;
}
